function _sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _slicedToArray(arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return _sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var CACHE = {};
/**
 * Class representing an ESLintError.
 * @extends Error
 */

var ESLintError =
/*#__PURE__*/
function (_Error) {
  _inherits(ESLintError, _Error);

  function ESLintError() {
    _classCallCheck(this, ESLintError);

    return _possibleConstructorReturn(this, (ESLintError.__proto__ || Object.getPrototypeOf(ESLintError)).apply(this, arguments));
  }

  return ESLintError;
}(Error);

var initOptions = function initOptions(opts) {
  var _opts$fix = opts.fix,
      fix = _opts$fix === void 0 ? false : _opts$fix,
      _opts$formatter = opts.formatter,
      formatter = _opts$formatter === void 0 ? 'stylish' : _opts$formatter,
      _opts$eslintPath = opts.eslintPath,
      eslintPath = _opts$eslintPath === void 0 ? 'eslint' : _opts$eslintPath,
      _opts$failOnError = opts.failOnError,
      failOnError = _opts$failOnError === void 0 ? true : _opts$failOnError,
      _opts$failOnWarning = opts.failOnWarning,
      failOnWarning = _opts$failOnWarning === void 0 ? false : _opts$failOnWarning;
  return {
    fix: fix,
    formatter: formatter,
    eslintPath: eslintPath,
    failOnError: failOnError,
    failOnWarning: failOnWarning
  };
};

var getESLint = function getESLint(options) {
  return require(options.eslintPath);
};

var initCli = function initCli(options) {
  var eslint = getESLint(options);
  return new eslint.CLIEngine({
    fix: options.fix
  });
};

var execLint = function execLint(cli, input, resourcePath) {
  return cli.executeOnText(input, resourcePath, true);
};

var fixLint = function fixLint(report, options) {
  if (options.fix && report.results && report.results[0].output) {
    var eslint = getESLint(options);
    eslint.CLIEngine.outputFixes(report);
  }
};

var getFirstFailingMessage = function getFirstFailingMessage(options, results, onlyErrors) {
  if (onlyErrors) {
    var eslint = getESLint(options);
    results = eslint.CLIEngine.getErrorResults(results);
  }

  var _results = results,
      _results2 = _slicedToArray(_results, 1),
      _results2$ = _results2[0],
      errorItem = _results2$ === void 0 ? {} : _results2$;

  var _ref = errorItem.messages || [],
      _ref2 = _slicedToArray(_ref, 1),
      message = _ref2[0];

  return message;
};

var visitor = function visitor() {
  return {
    visitor: {
      Program: function Program(path, state) {
        // ------------- Init cache ----------
        if (!CACHE.options) {
          CACHE.options = initOptions(state.opts);
        }

        if (!CACHE.cli) {
          CACHE.cli = initCli(CACHE.options);
        }

        var file = state.file;
        var opts = file.opts,
            code = file.code;
        var cli = CACHE.cli,
            options = CACHE.options;
        var report = execLint(cli, code, opts.filename);
        var isFailingError = options.failOnError && report.errorCount > 0;
        var isFailingWarning = options.failOnWarning && report.warningCount > 0; // auto-fix the code

        fixLint(report, options);

        if (isFailingError || isFailingWarning) {
          var formatter = cli.getFormatter(options.formatter);
          var formattedMessage = formatter(report.results);
          var failingMessage = getFirstFailingMessage(options, report.results, !isFailingWarning);
          var error = path.buildCodeFrameError(formattedMessage, ESLintError);

          if (failingMessage && error.loc && error.loc.line) {
            // Update error location
            error.loc.line = failingMessage.line;
            error.loc.column = failingMessage.column;
          }

          throw error;
        }
      }
    }
  };
};

module.exports = visitor;